@@ -12,45 +12,15 @@ from websockets.server import HTTPResponse
 from websockets.http import Headers
 
 from .types import Socket
+from .socket import TCPSocket
 from .utils import onlyfirst
+from .exceptions import WebSocksImplementationError
 
-logger: logging.Logger = logging.getLogger("websocks")
-
-
-class TCPSocket(Socket):
-    def __init__(self, reader: asyncio.StreamReader, writer: asyncio.StreamWriter):
-        self.r = reader
-        self.w = writer
-
-    @classmethod
-    async def create_connection(cls, host: str, port: int) -> "TCPSocket":
-        """create a TCP socket"""
-        r, w = await asyncio.open_connection(host=host, port=port)
-        return TCPSocket(r, w)
-
-    async def recv(self, num: int = 4096) -> bytes:
-        data = await self.r.read(num)
-        return data
-
-    async def send(self, data: bytes) -> int:
-        self.w.write(data)
-        await self.w.drain()
-        return len(data)
-
-    async def close(self) -> None:
-        self.w.close()
-        try:
-            await self.w.wait_closed()
-        except ConnectionError:
-            pass
-
-    @property
-    def closed(self) -> bool:
-        return self.w.is_closing()
+logger: logging.Logger = logging.getLogger(__name__)
 
 
 async def bridge(alice: Socket, bob: Socket) -> None:
-    async def _bridge(sender: Socket, receiver: Socket) -> None:
+    async def _(sender: Socket, receiver: Socket) -> None:
         while True:
             data = await sender.recv()
             if not data:
@@ -58,7 +28,7 @@ async def bridge(alice: Socket, bob: Socket) -> None:
             await receiver.send(data)
 
     try:
-        await onlyfirst(_bridge(alice, bob), _bridge(bob, alice))
+        await onlyfirst(_(alice, bob), _(bob, alice))
     except OSError:
         pass
 
@@ -82,7 +52,8 @@ class Server:
                 websocks_has_closed = False
 
                 data = await sock.recv()
-                assert isinstance(data, str)
+                if not isinstance(data, str):
+                    raise WebSocksImplementationError()
                 request = json.loads(data)
                 try:
                     remote = await TCPSocket.create_connection(
@@ -110,9 +81,10 @@ class Server:
                         msg = await sock.recv()
                         if isinstance(msg, str):
                             break
-                    assert json.loads(msg)["STATUS"] == "CLOSED"
+                    if json.loads(msg)["STATUS"] != "CLOSED":
+                        raise WebSocksImplementationError()
 
-        except (AssertionError, KeyError):
+        except (WebSocksImplementationError, KeyError):
             ...  # websocks implemented error
         except websockets.exceptions.ConnectionClosed:
             ...
@@ -135,11 +107,11 @@ class Server:
     async def run_server(self) -> typing.NoReturn:
         async with websockets.serve(
             self._link, host=self.host, port=self.port, process_request=self.handshake
-        ) as server:
-            logger.info(f"Websocks Server serving on {self.host}:{self.port}")
+        ):
+            logger.info(f"WebSocks Server serving on {self.host}:{self.port}")
 
             def termina(signo, frame):
-                logger.info(f"Websocks Server has closed.")
+                logger.info("WebSocks Server has closed.")
                 raise SystemExit(0)
 
             signal.signal(signal.SIGINT, termina)
